---------------------------------------------------------------------------
-- Location Mod v1.1 for Counter-Strike 2D                               --
--                                                                       --
-- Created by ColdX (USGN: 10055) - October 14, 2010                     --
-- http://www.unrealsoftware.de/profile.php?userid=10055                 --
--                                                                       --
-- This Lua script for Counter-Strike 2D displays a player's country and --
-- USGN ID whenever they join the server.                                --
---------------------------------------------------------------------------

-- WebNet "IPToCountry.csv" (http://software77.net/geo-ip/?DL=2&;x=Download) Parsing Routine

webnet = {}

function webnet:new(name,strict)
        local w = setmetatable({},{__index=webnet})
    if type(name) == 'string' then 
        local t,e = w:load(name,strict)
        if t == nil then w = nil; return nil,e end
    end
    return w
end

function webnet:load(name,strict)
    local last = -1
    local ib,ie,ia,_,co,cou,coun
    local ibn, ien
    local iptab,cotab = {},{}
    local f = io.open(name,"rt")
    
    -- Is the file actually there?
    if f == nil then return nil,"file not found" end
    
    -- Iterate through the file.
    for str in f:lines() do
        -- Try to split the line into fields.
        ib,ie,ia,_,co,cou,coun = 
                str:match('"(%d+)","(%d+)","(%a+)","(%d+)","(%u%u)","(%u%u%u?)","(.*)"')
        -- Check that it was a well-formed line.
        if ib == nil or ie == nil or co == nil then
            -- Unparseable lines are ignored unless they start with '"'.
            -- This prevents new-format lines getting ignored for ever.
            -- Note we must skip lines where co=nil lest we return it.
            if str:sub(1,1) == '"' then return nil,"bad line in file",str end
        else
            -- Parseable lines belong in the table.
            ibn, ien = tonumber(ib), tonumber(ie)
            if ibn <= last or ien < ibn then
                -- Don't accept this record if it's out-of-order.
                -- (See comments above about repeated ranges in real data.)
                if strict then return nil,"ip data out of order" end
            else
                -- If this record doesn't follow on from the last, fill the 'gap',
                if ibn ~= last+1 then
                    iptab[#iptab+1] = last+1
                    cotab[#cotab+1] = "-- UNALLOCATED"
                end
                -- and add this record to the (sorted) lists.
                -- Note: Lua internalises strings so the regular repetition
                -- of common country names doesn't waste memory.
                iptab[#iptab+1] = ibn
                cotab[#cotab+1] = co..' '..coun
                last = ien
            end
        end
    end    
    
    -- Add a catchall record (doesn't matter if out of 32-bit range).
    iptab[#iptab+1] = last+1
    cotab[#cotab+1] = "-- UNALLOCATED"
    
    self.iptab, self.cotab = iptab, cotab
    return self
end

function webnet:lookup(ip)
    -- Validate and convert the IP parameter.
    if type(ip) == 'number' then 
       if ip<0 or ip>=2^32 then return nil,"bad ip number" end
    elseif type(ip) == 'string' then
       local a,b,c,d = ip:match('(%d+).(%d+).(%d+).(%d+)')
       if a==nil or b==nil or c==nil or d==nil then return nil, "bad ip string" end
       a,b,c,d = a+0,b+0,c+0,d+0
       if a<0 or b<0 or c<0 or d<0 then return nil,"bad ip string" end
       if a>255 or b>255 or c>255 or d>255 then return nil,"bad ip string" end
       ip = a*2^24 + b*2^16 + c*2^8 + d
    else
       return nil, "bad ip parameter"
    end
    
    -- Check that IP number is inside our table's coverage.
    local bot,top = 1,#self.iptab
    if ip <  self.iptab[bot] then return "-- ","UNALLOCATED" end
    if ip >= self.iptab[top] then return "-- ","UNALLOCATED" end
    
    -- Now binary chop the table until we find the country.
    -- This should never take more than log-base-2(tablesize) chops.
    -- Lap and lim are there just in case, to ensure no infinite loops.
    local lap,lim = 0,(math.log(#self.iptab)/math.log(2))*1.5
    repeat
        lap = lap+1
        local mid = bot + math.floor((top-bot)/2)
        -- See if we've found the answer.
        if ip >= self.iptab[mid] and ip < self.iptab[mid+1] then
            return self.cotab[mid]:match('(%C%C) (.*)')
        end
        -- If not, chop off the half it can't be in.
        if ip < self.iptab[mid] then top = mid else bot = mid+1 end
    until bot >= top or lap > lim
    
    -- This isn't supposed to happen unless the table is corrupt.
    return nil,"lookup error"
end

-- Location Mod Routine

w = webnet.new()
e,e_msg,s = w:load('Sys/IPToCountry.csv')

addhook("join","location_mod")
function location_mod(id)
        iso,country_name = w:lookup(player(id,"ip"))
        if country_name=="Reserved" then
                -- country_name="Reserved"
                -- country_name="a local LAN"
                country_name="a local LAN"
        else
                country_name=country_name
        end
        if player(id,"usgn")==0 then
                -- msg("000255255"..player(id,"name").." (USGN ID: NONE) has joined the game from "..country_name.."!")
                msg("000255255"..player(id,"name").." (USGN ID: NONE) has joined the game from "..country_name.."!")
        else
                -- msg("000255255"..player(id,"name").." (USGN ID: "..player(id,"usgn")..") has joined the game from "..country_name.."!")
                msg("000255255"..player(id,"name").." (USGN ID: "..player(id,"usgn")..") has joined the game from "..country_name.."!")
        end
end
